@@ -1,6 +1,6 @@
 # run-lisp-tests
 #
-# Copyright (c) 2009-2011 Free Software Foundation, Inc.
+# Copyright (c) 2009-2012 Free Software Foundation, Inc.
 #
 # This file is part of GNU Zile.
 #
@@ -30,7 +30,7 @@ use File::Path;
 # N.B. Tests that use execute-kbd-macro must note that keyboard input
 # is only evaluated once the script has finished running.
 
-# srcdir and builddir are defined in the environment for a build
+# The following are defined in the environment for a build
 my $abs_srcdir = $ENV{abs_srcdir} || `pwd`;
 chomp $abs_srcdir;
 my $srcdir = $ENV{srcdir} || ".";
@@ -41,9 +41,25 @@ my $zile_fail = 0;
 my $emacs_pass = 0;
 my $emacs_fail = 0;
 
-# If TERM is not set to a terminal type, choose a default
-if (-z $ENV{TERM} || $ENV{TERM} eq "unknown") {
-  $ENV{TERM} = "vt100";
+my @zile_cmd = ("$builddir/src/zile");
+unshift @zile_cmd, (split ' ', $ENV{VALGRIND}) if $ENV{VALGRIND};
+
+sub run_test {
+  my ($test, $name, $editor_name, $edit_file, @args) = @_;
+  copy("$srcdir/tests/test.input", $edit_file);
+  chmod 0644, $edit_file;
+  if (system(@args) == 0) {
+    if (system("diff", "$test.output", $edit_file) == 0) {
+      unlink $edit_file, "$edit_file~";
+      return 1;
+    } else {
+      print STDERR "$editor_name failed to produce correct output for test `$name'\n";
+      return 0;
+    }
+  } else {
+    print STDERR "$editor_name failed to run test `$name' with error code $?\n";
+    return 0;
+  }
 }
 
 for my $test (@ARGV) {				# ../tests/zile-only/backward_delete_char.el
@@ -53,43 +69,24 @@ for my $test (@ARGV) {				# ../tests/zile-only/backward_delete_char.el
   $edit_file =~ s/^\Q$srcdir/$builddir/e;	# ./tests/zile-only/backward_delete_char.input
   my $lisp_file = "$test.el";
   $lisp_file =~ s/^\Q$srcdir/$abs_srcdir/e;
+  my @args = ("--quick", "--batch", "--no-init-file", $edit_file, "--load", $lisp_file);
 
   mkpath(dirname($edit_file));
 
-  my @args = ("--no-init-file", $edit_file, "--load", $lisp_file);
-
   if ($ENV{EMACSPROG}) {
-    copy("$srcdir/tests/test.input", $edit_file);
-    chmod 0644, $edit_file;
-    if (system($ENV{EMACSPROG}, "--quick", "--batch", @args) == 0) {
-      if (system("diff", "$test.output", $edit_file) == 0) {
-        $emacs_pass++;
-        unlink $edit_file, "$edit_file~";
-      } else {
-        print STDERR "Emacs $name failed to produce correct output\n";
-        $emacs_fail++;
-      }
+    if (run_test($test, $name, "Emacs", $edit_file, $ENV{EMACSPROG}, @args)) {
+      $emacs_pass++;
     } else {
-      print STDERR "Emacs $name failed to run with error code $?\n";
       $emacs_fail++;
+      rename $edit_file, "$edit_file-emacs";
+      rename "$edit_file~", "$edit_file-emacs~";
     }
   }
 
-  copy("$srcdir/tests/test.input", $edit_file);
-  chmod 0644, $edit_file;
-  my @zile_cmd = ("$builddir/src/zile");
-  unshift @zile_cmd, (split ' ', $ENV{VALGRIND}) if $ENV{VALGRIND};
-  if (system(@zile_cmd, @args) == 0) {
-    if (system("diff", "$test.output", $edit_file) == 0) {
-      $zile_pass++;
-      unlink $edit_file, "$edit_file~";
-    } else {
-      print STDERR "Zile $name failed to produce correct output\n";
-      $zile_fail++;
-    }
+  if (run_test($test, $name, "Zile", $edit_file, @zile_cmd, @args)) {
+    $zile_pass++;
   } else {
-    print STDERR "Zile $name failed to run with error code $?\n";
-    $zile_fail++
+    $zile_fail++;
   }
 }
 
